#include <pjlib.h>
#include <pjmedia.h>

#define CLOCK_RATE	16000
#define NSAMPLES	320

//#define TEST_WSOLA
#define TEST_EC

int main()
{
    pj_caching_pool cp;
    pj_pool_t *pool;
    pjmedia_port *src_port;
    pjmedia_port *dst_port;
#ifdef TEST_WSOLA
    pjmedia_wsola *wsola;
#elif defined(TEST_EC)
    pjmedia_echo_state *ec;
#else
    pjmedia_delay_buf *db;
#endif
    pjmedia_frame frame;
    short framebuf[NSAMPLES];
    pjmedia_tone_desc tone;
    unsigned i;
    pj_bool_t prevlost = PJ_FALSE;
    pj_status_t status;

    status = pj_init();
    pj_assert(status == PJ_SUCCESS);

    pj_caching_pool_init(&cp, NULL, 0);
    pool = pj_pool_create(&cp.factory, "", 1000, 1000, NULL);

    status = pjmedia_tonegen_create(pool, CLOCK_RATE, 1, NSAMPLES, 16, 0, &src_port);
    pj_assert(status == PJ_SUCCESS);

    tone.freq1 = 440;
    tone.freq2 = 480;
    tone.on_msec = 10000;
    tone.off_msec = 0;
    tone.volume = 0;
    tone.flags = 0;

    status = pjmedia_tonegen_play(src_port, 1, &tone, PJMEDIA_TONEGEN_LOOP);
    pj_assert(status == PJ_SUCCESS);

#ifdef TEST_WSOLA
    status = pjmedia_wsola_create(pool, CLOCK_RATE, NSAMPLES, 1, 0, &wsola);
#elif defined(TEST_EC)
    status = pjmedia_echo_create(pool, CLOCK_RATE, NSAMPLES, 200, 100, 0, &ec);
    pj_assert(status == PJ_SUCCESS);
#else
    status = pjmedia_delay_buf_create(pool, "", CLOCK_RATE, NSAMPLES, 1, 400, 0, &db);
#endif
    pj_assert(status == PJ_SUCCESS);

    status = pjmedia_wav_writer_port_create(pool, "expanded.wav", CLOCK_RATE, 1, NSAMPLES, 16, 0, 0, &dst_port);
    pj_assert(status == PJ_SUCCESS);

    frame.buf = framebuf;
    frame.size = NSAMPLES * 2;
    frame.type = PJMEDIA_FRAME_TYPE_AUDIO;

    for (i=0; i<5000; ++i) {
#ifdef TEST_WSOLA
	if ((i%50) != 0) {
	    status = status = pjmedia_port_get_frame(src_port, &frame);
	    pj_assert(status == PJ_SUCCESS);

	    status = pjmedia_wsola_save(wsola, framebuf, PJ_FALSE);
	    pj_assert(status == PJ_SUCCESS);
	    prevlost = PJ_FALSE;
	} else {
	    status = pjmedia_wsola_generate(wsola, framebuf);
	    pj_assert(status == PJ_SUCCESS);
	    prevlost = PJ_TRUE;
	}
#elif defined(TEST_EC)
	if (i==0 || (i%50) != 0) {
	    status = status = pjmedia_port_get_frame(src_port, &frame);
	    pj_assert(status == PJ_SUCCESS);

	    status = pjmedia_echo_playback(ec, framebuf);
	    pj_assert(status == PJ_SUCCESS);

	    status = pjmedia_port_put_frame(dst_port, &frame);
	    pj_assert(status == PJ_SUCCESS);

	    status = pjmedia_echo_capture(ec, NULL, 0);
	    pj_assert(status == PJ_SUCCESS);

	} else {
	    status = pjmedia_echo_capture(ec, NULL, 0);
	    pj_assert(status == PJ_SUCCESS);
	}

#else
	if ((i%50) != 0) {
	    status = status = pjmedia_port_get_frame(src_port, &frame);
	    pj_assert(status == PJ_SUCCESS);

	    status = pjmedia_delay_buf_put(db, framebuf);
	    pj_assert(status == PJ_SUCCESS);
	}

	status = pjmedia_delay_buf_get(db, framebuf);
	pj_assert(status == PJ_SUCCESS);
#endif
    }
    pjmedia_port_destroy(dst_port);
    pjmedia_port_destroy(src_port);
#ifdef TEST_WSOLA
    pjmedia_wsola_destroy(wsola);
#elif defined(TEST_EC)
    pjmedia_echo_destroy(ec);
#else
    pjmedia_delay_buf_destroy(db);
#endif

    pj_pool_release(pool);
    pj_caching_pool_destroy(&cp);
    pj_shutdown();

    return 0;
}
